<?php

namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AlteracaoAutomaticaDeParticularidadeController extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $environment;

    public function __construct($crudService, $parameterBag, $environment, $dataSourceManager, $entityManager, $connection, $supUtilities)
    {
        parent::__construct($crudService, $parameterBag);
        $this->crudService       = $crudService;
        $this->environment       = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager     = $entityManager;
        $this->connection        = $connection;
        $this->supUtilities      = $supUtilities;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->addCondition("P_CDOPERADOR", $this->environment->getCdoperador());
        $conditions = $filterCriteria->getConditions();
        $dataset = $this->crudService->getFilteredDataSet("alteracaoautomaticadeparticularidade", $conditions)->getRows();
        $response->addDataSet(new DataSet("resp", $dataset));
    }

    public function confirmarWidget(Request\Row $request, Response $response)
    {
        $dataset       = $request->getRow();
        $arrayPosition = $dataset[0];
        $arrayGrid     = $dataset[1];

        foreach ($arrayGrid as &$row) {
            if ($row['CDTIPPART'] == '1' || $row['CDTIPPART'] == '2' || $row['CDTIPPART'] == '3') {
                $row['RESULT'] = $this->verificaMarca($arrayPosition, $row) ? '2' : '1';
            }
            if ($row['CDTIPPART'] == '4' || $row['CDTIPPART'] == '5' || $row['CDTIPPART'] == '6') {
                $row['RESULT'] = $this->verificaFornecedor($arrayPosition, $row) ? '4' : '3';
            }
            if ($row['CDTIPPART'] == '7' || $row['CDTIPPART'] == '8' || $row['CDTIPPART'] == '9') {
                $row['RESULT'] = $this->verificaMarcaFornecedor($arrayPosition, $row) ? '6' : '5';
            }
        }
        $response->addDataSet(new DataSet("resp", $arrayGrid));
    }

    public function sqlBrand($arrayPosition, $row)
    {
        $sql = "SELECT MARCA.CDMARCA, MARCA.NMMARCA FROM MARCA WHERE MARCA.CDMARCA = :P_CDMARCA AND EXISTS (SELECT 1 FROM MARCPRODCOMP WHERE MARCPRODCOMP.CDMARCA  = MARCA.CDMARCA AND MARCPRODCOMP.CDPRMARCOMPL = :P_CDPRODUTO)";
        $params =
            [
                'P_CDMARCA'   =>  $arrayPosition['CDMARCA'],
                'P_CDPRODUTO' =>  $row['CDPRODUTO']
            ];
        $std = $this->supUtilities->crud($sql, $params);
        if (empty($std) === false) {
            return true;
        } else {
            return false;
        }
    }

    public function sqlForneced($arrayPosition, $row)
    {
        $selectForneced = "SELECT FORNECEDOR.CDFORNECED FROM FORNECEDOR WHERE FORNECEDOR.CDFORNECED = :P_CDFORNECED AND EXISTS (SELECT 1 FROM PRODFORNCOMP WHERE PRODFORNCOMP.CDFORNECED = FORNECEDOR.CDFORNECED AND PRODFORNCOMP.CDPRODCOMPL = :P_CDPRODUTO)";
        $parametros =
            [
                'P_CDFORNECED'   =>  $arrayPosition['CDFORNECED'],
                'P_CDPRODUTO'    =>  $row['CDPRODUTO']
            ];
        $selectFunction = $this->supUtilities->crud($selectForneced, $parametros);
        if (empty($selectFunction) === false) {
            return true;
        } else {
            return false;
        }
    }

    public function verificaMarca($arrayPosition, $row)
    {
        $verification = $this->sqlBrand($arrayPosition, $row);
        if ($verification === true) {
            $queryBrandParticularity = "UPDATE PARTFILI SET CDMARCA    = :P_CDMARCA WHERE CDFILIAL  = :P_CDFILIAL AND CDPRODUTO = :P_CDPRODUTO AND CDTIPPART  = :P_CDTIPPART";
            $param =
                [
                    'P_CDMARCA'   => $arrayPosition['CDMARCA'],
                    'P_CDPRODUTO' => $row['CDPRODUTO'],
                    'P_CDTIPPART' => $row['CDTIPPART'],
                    'P_CDFILIAL'  => $row['CDFILIAL']
                ];
            $this->supUtilities->crud($queryBrandParticularity, $param);
            return true;
        } else {
            return false;
        }
    }

    public function verificaFornecedor($arrayPosition, $row)
    {
        $verificationProvider = $this->sqlForneced($arrayPosition, $row);
        if ($verificationProvider === true) {
            $queryProviderParticularity = "UPDATE PARTFILI SET CDFORNECED = :P_CDFORNECED WHERE CDFILIAL  = :P_CDFILIAL AND CDPRODUTO = :P_CDPRODUTO AND CDTIPPART  = :P_CDTIPPART";
            $parametro =
                [
                    'P_CDFORNECED'   => $arrayPosition['CDFORNECED'],
                    'P_CDPRODUTO'    => $row['CDPRODUTO'],
                    'P_CDTIPPART'    => $row['CDTIPPART'],
                    'P_CDFILIAL'     => $row['CDFILIAL']
                ];
            $this->supUtilities->crud($queryProviderParticularity, $parametro);
            return true;
        } else {
            return false;
        }
    }

    public function verificaMarcaFornecedor($arrayPosition, $row)
    {
        $verification = $this->sqlBrand($arrayPosition, $row);
        $verificationProvider = $this->sqlForneced($arrayPosition, $row);
        if ($verification === true && $verificationProvider === true) {
            $queryBrandProviderParticularity = "UPDATE PARTFILI SET CDMARCA    = :P_CDMARCA, CDFORNECED = :P_CDFORNECED WHERE CDFILIAL  = :P_CDFILIAL AND CDPRODUTO = :P_CDPRODUTO AND CDTIPPART  = :P_CDTIPPART";
            $parametrosMarcaForneced =
                [
                    'P_CDFORNECED'   => $arrayPosition['CDFORNECED'],
                    'P_CDMARCA'      => $arrayPosition['CDMARCA'],
                    'P_CDPRODUTO'    => $row['CDPRODUTO'],
                    'P_CDTIPPART'    => $row['CDTIPPART'],
                    'P_CDFILIAL'     => $row['CDFILIAL']
                ];
            $this->supUtilities->crud($queryBrandProviderParticularity, $parametrosMarcaForneced);
            return true;
        } else {
            return false;
        }
    }
}
