<?php

namespace Modules\Cotacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Modules\Cotacao\Services\PedidoColetaService;
use Modules\Cotacao\Modules\Mercadum\Controllers\IntegracaoMercadumController;
use PhpOffice\PhpSpreadSheet\Spreadsheed;
use PhpOffice\PhpSpreadSheet\Writer\Xlsx;
use ZipArchive;

class PedidoColetaController extends \Zeedhi\Framework\Controller\Report
{

    protected $entityManager;
    protected $smtpUtil;
    protected $supUtilities;
    protected $reportService;
    protected $qr2Strategy;
    protected $environment;
    protected $service;
    protected $birtStrategy;
    protected $integracaoMercadum;
    protected $newCode;
    protected $crudService;
    protected $cookiePedidoColetaInternet = 'PEDIDO_COTETA_INTERNET';

    public function __construct($reportService, $qr2Strategy, $entityManager, $smtpUtil, $supUtilities, PedidoColetaService $service, $environment, $newCode, $connection, $birtStrategy, IntegracaoMercadumController $integracaoMercadum, $crudService)
    {
        $this->entityManager = $entityManager;
        $this->smtpUtil = $smtpUtil;
        $this->supUtilities = $supUtilities;
        $this->reportService = $reportService;
        $this->qr2Strategy = $qr2Strategy;
        $this->service = $service;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->birtStrategy = $birtStrategy;
        $this->integracaoMercadum = $integracaoMercadum;
        $this->crudService = $crudService;
    }
    
    public function getDataPedidoColeta(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $comprador = $this->verifyComprador($row);
            $datasetEmail = [];
            $exists = [];
            
            
            if(empty($comprador['message']) && empty($comprador['comprador'])) {
                $connection = $this->entityManager->getConnection();
                $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.GETDATAPEDIDOCOLETA');

                $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['CDFILIAL']));
                $procedure->addParam(new Param('P_CDFILIPROC',   Param::PARAM_INPUT, $this->environment->getCdFilial()));
                $procedure->addParam(new Param('P_CDCOMPRADOR',  Param::PARAM_INPUT, $row['CDCOMPRADOR']));
                $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['CDPRODINI']));
                $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['CDPRODFIN']));
                $procedure->addParam(new Param('P_CDFORNECED',   Param::PARAM_INPUT, $row['CDFORNECED']));
                $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT, $row['CDREGIAO']));
                $procedure->addParam(new Param('P_CDMARCA',      Param::PARAM_INPUT, $row['CDMARCA']));
                $procedure->addParam(new Param('P_DTINI',        Param::PARAM_INPUT, $row['DTINI']));
                $procedure->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT, $row['DTFIN']));
                $procedure->addParam(new Param('P_TIPOPEDIDO',   Param::PARAM_INPUT, $row['TIPOPEDIDO']));
                $procedure->addParam(new Param('P_NRCOLEPRECO',  Param::PARAM_INPUT, $row['NRCOLEPRECO']));
                $procedure->addParam(new Param('P_NRCOMPRA',     Param::PARAM_INPUT, $row['NRCOMPRA']));
                $procedure->addParam(new Param('P_CDLEILAO',     Param::PARAM_INPUT, $row['CDLEILAO']));
                $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $row['NRSEQPRELIST']));
                $procedure->addParam(new Param('P_CDOPERLANC',   Param::PARAM_INPUT, $this->environment->getCdoperador()));
                $procedure->addParam(new Param('P_NRORG',        Param::PARAM_INPUT, $this->environment->getNrOrg()));
                $procedure->addParam(new Param('P_NRPEDCOL',     Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
                $procedure->addParam(new Param('P_ERROR',        Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 20));
                $result = $procedure->execute();
                
                $datasetForne = array_unique($this->getFornecedorPedidoColeta($result), SORT_REGULAR);
                $datasetEmail = $this->getFornecedorEmail($datasetForne);
                $exists = $this->existePedidoColeta($result, $row);
            }
            
            $response->addDataSet(new DataSet("data", array('pedidos' => $datasetEmail, 'exists' => $exists, 'comprador' => $comprador)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
    }
    
    private function verifyComprador($row)
    {
        try {
            
            $cdoperador = $this->environment->getCdoperador();
            
            $operadorRelac = $this->getOperadorRelacionado($row);
            $relacionado = array_filter($operadorRelac, function($oper) use ($cdoperador) {
                return $oper['CDOPERADOR'] == $cdoperador;
            });
            
            $message = empty($relacionado) ? 'NO_OPERATOR_RELATED_BUYER' : '';
            
            $compradorNaoRelacionado = array();
            if(empty($message)) {
                $compradorNaoRelacionado = $this->getOperadorNaoRelacionado($row, $cdoperador);
            }
            
            return array('message' => $message, 'comprador' => $compradorNaoRelacionado);
        
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function joinArrayValue($array, $key, $return = null)
    {
        return isset($array[$key]) && is_array($array[$key]) ? join('_', $array[$key]) : $return;
    }
    
    private function getOperadorRelacionado($param)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDCOMPRADOR',
                    'operator' => '=',
                    "value" => $param['CDCOMPRADOR']
                )
            );
            
            return $this->crudService->getFilteredDataSet("operadorRelacionado", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getOperadorNaoRelacionado($param, $cdoperador)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDCOMPRADOR',
                    'operator' => '=',
                    "value" => $param['CDCOMPRADOR']
                ),
                array(
                    'columnName' => 'P_CDOPERADOR',
                    'operator' => '=',
                    "value" => $cdoperador
                )
            );
            
            return $this->crudService->getFilteredDataSet("operadorNaoRelacionado", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getFornecedorPedidoColeta($param)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_NRPEDCOL',
                    'operator' => '=',
                    "value" => $param['P_NRPEDCOL']
                )
            );
            
            return $this->crudService->getFilteredDataSet("fornecedorPedidoColeta", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getFornecedorEmail($param)
    {
        try {
            $nrpedcol = $param && count($param) ? $param[0]['NRPEDCOL'] : '';
            $cdforneced = $this->getUniqueKey($param, 'CDFORNECED');
            
            $conditions = array(
                array(
                    'columnName' => 'P_NRPEDCOL',
                    'operator' => '=',
                    "value" => $nrpedcol
                ),
                array(
                    'columnName' => 'P_CDFORNECED',
                    'operator' => '=',
                    "value" => $cdforneced
                )
            );

            return $this->crudService->getFilteredDataSet("fornecedorEmail", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getPedidoColeta($param)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_NRPEDCOL',
                    'operator' => '=',
                    "value" => $param['P_NRPEDCOL']
                )
            );

            return $this->crudService->getFilteredDataSet("tmpPedidoColeta", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getUniqueKey($array, $key)
    {
        if(isset($array) && is_array($array) && count($array)) {
            return join("_", array_unique(array_map(function($row) use ($key) {
                return $row[$key];
            }, $array)));
        } else {
            return null;    
        }
    }
    
    private function existePedidoColeta($param, $row)
    {
        try {
            
            $pedidos = $this->getPedidoColeta($param);
            
            $cdforneced = $this->getUniqueKey($pedidos, 'CDFORNECED');
            $cdregiao = $this->getUniqueKey($pedidos, 'CDREGIAO');
            $cdoperador = $this->getUniqueKey($pedidos, 'CDOPERADOR');
            
            $conditions = array(
                array(
                    'columnName' => 'P_CDREGIAO',
                    'operator' => '=',
                    "value" => $cdregiao
                ),
                array(
                    'columnName' => 'P_CDFORNECED',
                    'operator' => '=',
                    "value" => $cdforneced
                ),
                array(
                    'columnName' => 'P_CDCOMPRADOR',
                    'operator' => '=',
                    "value" => $cdoperador
                ),
                array(
                    'columnName' => 'P_DTFIN',
                    'operator' => '=',
                    "value" => $row['DTFIN']
                ),
                array(
                    'columnName' => 'P_DTINI',
                    'operator' => '=',
                    "value" => $row['DTINI']
                )
            );

            return $this->crudService->getFilteredDataSet("existePedidoColeta", $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getMensagemEnvioEmail(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $msg = $this->supUtilities->getMensagemPadraoEmail('S');
            $cookie = $this->supUtilities->retornaCookieEmail($this->cookiePedidoColetaInternet);
        
            $response->addDataSet(new DataSet("data", array('message' => $msg, 'cookie' => $cookie)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function retornaCookieEmail(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        $cookie = $this->supUtilities->retornaCookieEmail($this->cookiePedidoColetaInternet);

        $response->addMessage(new DTO\Response\Message($cookie));
    }
    
    public function verificaExistePedidoColeta(DTO\Request\Row $request, DTO\Response $response)
    {

        $row = $request->getRow();

        try {
            $query = "SELECT FORNECEDOR.CDFORNECED, FORMATMASK(FORNECEDOR.IDTPIJURFORN,FORNECEDOR.NRINSJURFORN) NRINSJURFORN, FORNECEDOR.NMFANTFORN, REGIAO.CDREGIAO || ' - ' || REGIAO.NMREGIAO NMREGIAO, NRPEDIDOCOL, DTPEDCOLETA, OPERADOR.CDOPERADOR || ' - ' || OPERADOR.NMOPERADOR NMOPERADOR FROM PEDCOLETA, FORNECEDOR, REGIAO, OPERADOR WHERE (( INSTR(:CDREGIAO,PEDCOLETA.CDREGIAO) > 0 ) OR (:CDREGIAO = 'T')) AND (( INSTR(:CDFORNECED,FORNECEDOR.CDFORNECED) > 0 ) OR (:CDFORNECED = 'T')) AND (( INSTR(:CDCOMPRADOR,PEDCOLETA.CDOPERADOR) > 0 ) OR (:CDCOMPRADOR = 'T')) AND ( DTINIPERCOT < TRUNC(TO_DATE(:DTFINPERCOT,'DD/MM/YYYY')-1)) AND (DTFINPERCOT > TRUNC(TO_DATE(:DTINIPERCOT,'DD/MM/YYYY')-1)) AND ( PEDCOLETA.CDFORNECED = FORNECEDOR.CDFORNECED ) AND ( PEDCOLETA.CDREGIAO = REGIAO.CDREGIAO ) AND ( PEDCOLETA.CDOPERADOR = OPERADOR.CDOPERADOR ) ORDER BY FORNECEDOR.NRINSJURFORN";
            $params = [
                "CDREGIAO" => $row['CDREGIAO'],
                "CDFORNECED" => $row['CDFORNECED'],
                "CDCOMPRADOR" => $row['CDCOMPRADOR'],
                "DTINIPERCOT" => $row['DTINIPERCOT'],
                "DTFINPERCOT" => $row['DTFINPERCOT']
            ];

            $existePedido = $this->supUtilities->crud($query, $params);

            $response->addMessage(new DTO\Response\Message($existePedido));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function pedcolPrevcons($row)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.SAVEPEDCOL_PREVCONS');
            
            $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT,  $row['CDFILIAL']));
            $procedure->addParam(new Param('P_PRODINI',   Param::PARAM_INPUT,  $row['PRODINI']));
            $procedure->addParam(new Param('P_PRODFIN',   Param::PARAM_INPUT,  $row['PRODFIN']));
            $procedure->addParam(new Param('P_MARCA',     Param::PARAM_INPUT,  $row['MARCA']));
            $procedure->addParam(new Param('P_COMPRADOR', Param::PARAM_INPUT,  $row['COMPRADOR']));
            $procedure->addParam(new Param('P_REGIAO',    Param::PARAM_INPUT,  $row['REGIAO']));
            $procedure->addParam(new Param('P_FORNECED',  Param::PARAM_INPUT,  $row['FORNECED']));
            $procedure->addParam(new Param('P_DTINI',     Param::PARAM_INPUT,  $row['DTINI']));
            $procedure->addParam(new Param('P_DTFIN',     Param::PARAM_INPUT,  $row['DTFIN']));
            $procedure->addParam(new Param('P_NRORG',     Param::PARAM_INPUT,  $this->environment->getNrOrg()));
            $procedure->addParam(new Param('P_NRPEDCOL',  Param::PARAM_INPUT,  $row['NRPEDCOL']));
            
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function savePedidoColeta($row)
    {
        try {
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PEDIDO_COLETA.SAVEPEDIDOCOLETA');
            
            $procedure->addParam(new Param('P_NRPEDCOL',       Param::PARAM_INPUT,  $row['NRPEDCOL']));
            $procedure->addParam(new Param('P_CDFILIAL',       Param::PARAM_INPUT,  $row['CDFILIAL']));
            $procedure->addParam(new Param('P_CDFILIPROC',     Param::PARAM_INPUT,  $this->environment->getCdFilial()));
            $procedure->addParam(new Param('P_QTMESES',        Param::PARAM_INPUT,  $row['QTMESES']));
            $procedure->addParam(new Param('P_DTEXPIRAPEDCOL', Param::PARAM_INPUT,  $row['DTEXPIRAPEDCOL']));
            $procedure->addParam(new Param('P_IDPEDGERALREG',  Param::PARAM_INPUT,  $row['IDPEDGERALREG']));
            $procedure->addParam(new Param('P_IDPEDGERALFORN', Param::PARAM_INPUT,  $row['IDPEDGERALFORN']));
            $procedure->addParam(new Param('P_IDCURVAABCAF',   Param::PARAM_INPUT,  $row['IDCURVAABCAF']));
            $procedure->addParam(new Param('P_IDCONSFORNAF',   Param::PARAM_INPUT,  $row['IDCONSFORNAF']));
            $procedure->addParam(new Param('P_ERROR',          Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR,  10));
            
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
        
    }
    
    private function getParamMercadum()
    {
        try {
            return $this->crudService->getFilteredDataSet("parammercadum", [])->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function savePedidoColetaWeb(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        try {
            
            if($row['BOACRESITEM'] == 'S') {
                $this->pedcolPrevcons($row);
            }
            
            $this->savePedidoColeta($row);
            $this->supUtilities->defineCookieEmail($this->cookiePedidoColetaInternet, $row['ORIGINAL_MESSAGE']);
            $request = $this->sendEmailPedidoColetaWeb($row);
            
            /*
             * Retirar esse comentário e verificar o controller resposável pela integração
             * com o mercadum
             *
            
            $parammercadum = $this->getParamMercadum();
            
            if (isset($parammercadum) && $parammercadum[0]['IDINTGPEDCOL'] == 'S') {
                $this->integracaoMercadum->integracaoMercadumPedidoColeta($row);
            }*/
            
            $response->addDataSet(new DataSet('request', $request));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    private function makeHtmlMessage($subject, $sentBy, $message)
    {
        return "<table>
                    <tr>
                        <td>"
                        . $subject . " 
                        </td>
                    </tr>
                            
                    <tr>
                        <td class='description'>
                            <br/>" . $message .
                        "</td>
                    </tr>
                    
                    <tr class='footer'>
                        <td>
                            <center>
                                <br> <p>" . $sentBy . " </p> 
                            </center>
                        </td>
                    </tr>
                </table>";
    }
    
    private function sendEmailPedidoColetaWeb($row)
    {
        if(isset($row['REQUESTS']) && is_array($row['REQUESTS'])) {
            foreach ($row['REQUESTS'] as &$request) {
                $message = $this->makeHtmlMessage($row['SUBJECT'], $row['SENT_BY'], $request['EMAIL_MESSAGE']);
                $messageSendMail = $this->supUtilities->messageSendMail($this->environment->getCdFilial(), $row['SUBJECT'], $message, 'PORTAL E-COMMERCE');
                try {
                    $result = $this->supUtilities->enviaEmail($row["LOGGED_EMAIL"], $row['SUBJECT'], $row["LOGGED_EMAIL"], $messageSendMail, $request['DSEMAILCOT']);
                    
                    $request['RESULT'] = 'SUCCESS';
                } catch (\Exception $e) {
                    $request['RESULT'] = 'ERROR';
                    $request['ERROR'] = $e->getMessage();
                }        
            }
        }
        
        return $row['REQUESTS'];
    }

    public function enviaEmailPedColetaWeb(DTO\Request\Row $request, DTO\Response $response)
    {

        $row = $request->getRow();

        $this->supUtilities->defineCookieEmail($this->cookiePedidoColetaInternet, $row['MENSAGEM_ORIGINAL']);

        $message = "<table>
                            <tr>
                                <td>"
            . $row['ASSUNTO'] . " 
                                </td>
                            </tr>
                            
                            <tr>
                                <td class='description'>
                                    <br/>" . $row['MENSAGEM'] .
            "</td>
                            </tr>
                            <tr class='footer'>
                                <td>
                                    <center>
                                        <br> <p> Email automático enviado pelo Suprimentos. </p> 
                                    </center>
                                </td>
                            </tr>
                        </table>";
        try {
            $assunto = $row['ASSUNTO'];
            $messageSendMail = $this->supUtilities->messageSendMail($row['FILIAL_LOGADA'], $assunto, $message, 'PORTAL E-COMMERCE');
            $this->supUtilities->enviaEmail($row["EMAIL_LOGADO"], $row['ASSUNTO'], $row['EMAILS'], $messageSendMail, $row['EMAILS']);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function defineCookieEmail(DTO\Request\Row $request, DTO\Response $response)
    {

        $row = $request->getRow();

        $this->supUtilities->defineCookieEmail($row['NOME'], $row['VALOR']);
    }
    
    public function verificaOperadorRelacionado(DTO\Request $request, DTO\Response $response)
    {
        try {
            /*$row = $request->getRow();
        
            $cdoperador = $this->environment->getCdoperador();
            
            $operadorRelac = $this->getOperadorRelacionado($row);
            $relacionado = array_filter($operadorRelac, function($oper) use ($cdoperador) {
                return $oper['CDOPERADOR'] == $cdoperador;
            });
            
            $message = empty($relacionado) ? 'NO_OPERATOR_RELATED_BUYER' : '';
            
            if(empty($message)) {
                $compradorNaoRelacionado = $this->getOperadorNaoRelacionado($row, $cdoperador);
            }*/
            
            $message = [];
            $compradorNaoRelacionado = [];
            $response->addDataSet(new DataSet("data", array('message' => $message, 'comprador' => $compradorNaoRelacionado)));
        
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
        
        /*var_dump($message, $compradorNaoRelacionado);die;
        
        
        if (isset($row['COMPRADOR'])) {
            foreach ($row['COMPRADOR'] as $comprador) {
                $operadoresRelac = $this->service->operadoresRelacionados($comprador);
                if (!self::operadorExistsInArray($operadoresRelac, $row['CDOPERADOR'])) {
                    $infoComp = $this->service->getNmComprador($comprador);
                    array_push($compNaoRelac, $infoComp);
                }
                $this->service->getNmComprador($comprador);
            }
        }

        $response->addDataSet(new DataSet("data", $compNaoRelac));*/
    }

    public function operadorExistsInArray($operArray, $operadorLog)
    {
        foreach ($operArray as $operador) {
            if ($operador['CDOPERADOR'] == $operadorLog) {
                return true;
            }
        }
        return false;
    }

    protected $reportMapping = array();

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        $cdfilial = $this->supUtilities->getCentralCompras()['CDFILCENTCMP'];
        $row['P_CDFILIPROC']    = $this->environment->getCdFilial();
        $row['P_CDFILIAL']      = $cdfilial;
        $row['P_CDOPERADOR']    = $this->environment->getCdoperador();

        $row['NRORG'] = $this->environment->getNrOrg();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

        if ($row['P_CDFORNECED'] != 'T') {
            $nrlistaForn = $this->criarListaValores($row['P_CDFORNECED']);
            $row['P_CDFORNECED'] = "_NRLISTA=" . $nrlistaForn;
        }

        if ($row['P_CDREGIAO'] != 'T') {
            $nrlistaReg = $this->criarListaValores($row['P_CDREGIAO']);
            $row['P_CDREGIAO'] = "_NRLISTA=" . $nrlistaReg;
        }

        if ($row['P_CDMARCA'] != 'T') {
            $nrlistaMarca = $this->criarListaValores($row['P_CDMARCA']);
            $row['P_CDMARCA'] = "_NRLISTA=" . $nrlistaMarca;
        }

        if ($row['P_CDCOMPRADOR'] != 'T') {
            $nrlistaComp = $this->criarListaValores($row['P_CDCOMPRADOR']);
            $row['P_CDCOMPRADOR'] = "_NRLISTA=" . $nrlistaComp;
        }

        $this->reportMapping[$this->stNomeRel] =
            array(
                'strategy' => 'BIRT',
                'parameterMapping' =>
                array(
                    "P_CDFILIAL"        => "P_CDFILIAL",
                    "P_NRCOLEPRECO"     => "P_NRCOLEPRECO",
                    "P_NRCOMPRA"        => "P_NRCOMPRA",
                    "P_CDREGIAO"        => "P_CDREGIAO",
                    "P_CDCOMPRADOR"     => "P_CDCOMPRADOR",
                    "P_CDFORNECED"      => "P_CDFORNECED",
                    "P_CDMARCA"         => "P_CDMARCA",
                    "P_DTINI"           => "P_DTINI",
                    "P_DTFIN"           => "P_DTFIN",
                    "P_DTINICIAL"       => "P_DTINICIAL",
                    "P_DTFINAL"         => "P_DTFINAL",
                    "P_CDPRODINI"       => "P_CDPRODINI",
                    "P_CDPRODFIN"       => "P_CDPRODFIN",
                    "P_TPRODUTO"        => "P_TPRODUTO",
                    "P_IMPOBS"          => "P_IMPOBS",
                    "P_DESCONSFORN"     => "P_DESCONSFORN",
                    "P_CDFILIPROC"      => "P_CDFILIPROC",
                    "P_ACOMPRAR"        => "P_ACOMPRAR",
                    "P_CDOPERADOR"      => "P_CDOPERADOR",
                    "NRORG"             => "P_NRORG"
                )
            );
    }

    protected function criarListaValores($lista)
    {
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_', $lista);
        try {
            foreach ($valores as $key) {
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insertLista($nrlista, $cdlista)
    {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );

        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }


    private function arrayKeyExists($array, $key)
    {
        return array_key_exists($key, $array) && $array && count($array);
    }

    private function concatArray($array, $key)
    {
        return self::arrayKeyExists($array, $key) ? join('_', $array[$key]) : 'T';
    }

    private function strPadRightProdFin($array)
    {
        return self::arrayKeyExists($array, 'CDPRODFIN')
            ? str_pad($array['CDPRODFIN'], 20, 'Z', STR_PAD_RIGHT)
            : str_pad('Z', 20, 'Z', STR_PAD_RIGHT);
    }

    private function setReportValues($row)
    {
        $row['P_NRCOLEPRECO']   = $row['NRCOLEPRECO'];
        $row['P_NRCOMPRA']      = '';
        $row['P_CDREGIAO']      = self::concatArray($row, 'REGIAO');
        $row['P_T_CDREGIAO']    = self::concatArray($row, 'REGIAO');
        $row['P_CDCOMPRADOR']   = self::concatArray($row, 'COMPRADOR');
        $row['P_T_CDCOMPRADOR'] = self::concatArray($row, 'COMPRADOR');
        $row['P_CDFORNECED']    = self::concatArray($row, 'FORNECEDOR');
        $row['P_T_CDFORNECED']  = self::concatArray($row, 'FORNECEDOR');
        $row['P_CDMARCA']       = self::concatArray($row, 'MARCA');
        $row['P_T_CDMARCA']     = self::concatArray($row, 'MARCA');
        $row['P_DTINI']         = $row['INTERVAL_VALIDADE']['start'];
        $row['P_DTFIN']         = $row['INTERVAL_VALIDADE']['end'];
        $row['P_DTINICIAL']     = $row['INTERVAL_VALIDADE']['start'];
        $row['P_DTFINAL']       = $row['INTERVAL_VALIDADE']['end'];
        $row['P_CDPRODINI']     = self::arrayKeyExists($row, 'CDPRODINI') ? $row['CDPRODINI'] : '0';
        $row['P_CDPRODFIN']     = self::strPadRightProdFin($row);
        $row['P_TPRODUTO']      = self::arrayKeyExists($row, 'CDPRODINI') ? ' ' : 'T';
        $row['P_IMPOBS']        = 'N';
        $row['P_DESCONSFORN']   = $row['DESCFORNPRI'];
        $row['P_ACOMPRAR']      = $row['TODOSPROD'];

        return $row;
    }

    public function sendReportToFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = self::setReportValues($request->getRow());
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $response);

        /**
         * 
         * 
         * 
         * To Do
         * 
         * 
         * 
         * envio de email
         * 
         * SELECT CDFORNECED, NVL(FORNECEDOR.DSMAILPEDCOLCOP, FORNECEDOR.DSMAILCOTCOP) DSMAILCOTCOP,
         * NVL(
         *  (SELECT DSEMAILFORN FROM TABLE(DADOS_FORN_EMAIL(FORNECEDOR.CDFORNECED, NULL, 'S', :NRORG))), 
         *  NVL( FORNECEDOR.DSEMAILPEDCOLFO, FORNECEDOR.DSEMAILCOT )) 
         * AS DSEMAILCOT
         * FROM FORNECEDOR
         * WHERE INSTR(:P_CDFORNECED, FORNECEDOR.CDFORNECED) > 0
         * 
         * 
         * 
         */

        $dadosEmail = $rows['email'];

        $messages = [];

        if (is_array($rows['relatorioData'])) {
            for ($i = 0; $i < count($rows['relatorioData']); $i++) {
                try {
                    $row = $rows['relatorioData'][$i];
                    $row['reportLabel'] = 'Relatório';

                    //$this->qr2Strategy->setQr2Type($dadosEmail['TYPE']);
                    //parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($row), "", "", ""), $response);

                    //if(self::verificaParametrosEnvioRelatorioExtra($row)){
                    $row['reportLabel'] = $reportParams['NAME'];
                    $row['__report_name'] = $reportParams['NAME'];
                    $this->birtStrategy->setReportFormat($reportParams['TYPE']);
                    parent::createRemoteReport(new DTO\Request\Row(new DTO\Row($reportFilters), "", "", ""), $response);
                    //}
                    self::enviarEmail($row, $dadosEmail, $response);
                    $this->globalReports = [];
                } catch (\Exception $e) {
                    $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
                }
            }
        }
    }

    public function exportaXlsx(DTO\Request $request, DTO\Response $response)
    {
        $conditions = $request->getParameters();
        $filial = $this->environment->getCdFilial();
        $emailDownloadParam;
        foreach ($conditions['dataset'] as $param) {
            if ($param['name'] == "EMAILDOWNLOADARQUIVO")
                $emailDownloadParam = $param['value'];
        }
        $parametrosEmail = array_filter($conditions['dataset'], function ($value) {
            return $value['name'] == "REMETENTE" || $value['name'] == "ASSUNTO" || $value['name'] == "MENSAGEM";
        });

        foreach ($parametrosEmail as $parametro) {
            if ($parametro['name'] == "REMETENTE")
                $remetente = $parametro['value'];
            else if ($parametro['name'] == "ASSUNTO")
                $assunto = $parametro['value'];
            else
                $mensagem = $parametro['value'];
        }

        $parametrosPedCol = array_filter($conditions['dataset'], function ($value) {
            return $value['name'] != "EMAILDOWNLOADARQUIVO" && $value['name'] != "REMETENTE" && $value['name'] != "ASSUNTO" && $value['name'] != "MENSAGEM";
        });

        $dataSet = $this->crudService->getFilteredDataSet('getPedColExporta', $this->service->filtroPedidoCol($parametrosPedCol))->getRows();

        $fornecedor = array_unique(array_map(function ($row) {
            return $row['CDFORNECED'];
        }, $dataSet));

        if (!$fornecedor) {
            $returnFilePath = array(
                "SEMDADOS"   => true,
            );
            $response->addDataSet(new DataSet('returnFilePath', $returnFilePath));
        }
        $nameZipArchive = "PedidodeColeta" . uniqid() . ".zip";
        foreach ($fornecedor as $dataFornecedor) {

            $params = array_filter($dataSet, function ($row) use ($dataFornecedor) {
                return $row['CDFORNECED'] == $dataFornecedor;
            });
            $comprador = array_unique(array_map(function ($row) {
                return $row['CDOPERADOR'];
            }, $params));
            foreach ($comprador as $dataComprador) {
                $paraams = array_filter($params, function ($row) use ($dataComprador) {
                    return $row['CDOPERADOR'] == $dataComprador;
                });
                $rowsXlsx = $this->montaDadosXlsx($paraams);
                if ($emailDownloadParam == "E") {
                    $nameZipArchive = "PedidodeColeta" . uniqid() . ".zip";
                    $returnFilePath = $this->geraXlsxPedCol($rowsXlsx, $dataComprador, $dataFornecedor, $nameZipArchive);
                    $emailFornecedor = $this->emailFornecedor($dataFornecedor);
                    if ($emailFornecedor != "") {
                        $mensagemFormatada = $this->setMnemonico($mensagem, reset($paraams));
                        $filial = $this->environment->getCdFilial();
                        $messageSendMail = $this->supUtilities->messageSendMail($filial, $assunto, $mensagemFormatada, 'PORTAL E-COMMERCE');
                        $arquivo = $returnFilePath["CAMINHOARQUIVOZIP"] . $returnFilePath["NOMEARQUIVOZIP"];
                        $name = $returnFilePath["NOMEARQUIVOZIP"];
                        $this->supUtilities->enviaEmail($remetente, $assunto, $emailFornecedor, $messageSendMail, $emailFornecedor, $arquivo, $name, "zip");
                        unlink($returnFilePath["CAMINHOARQUIVOZIP"] . $returnFilePath["NOMEARQUIVOZIP"]);
                    }
                } else {
                    $returnFilePath = $this->geraXlsxPedCol($rowsXlsx, $dataComprador, $dataFornecedor, $nameZipArchive);
                }
            }
        };
        $response->addDataSet(new DataSet('returnFilePath', $returnFilePath));
    }

    public function montaDadosXlsx($dataSet)
    {
        $rowsXlsx = array();
        foreach ($dataSet as $row) {
            $data = array(
                $row['CDREGIAO'],
                $row['NMREGIAO'],
                $row['VALIDADE_INI'],
                $row['VALIDADE_FIN'],
                $row['CDOPERADOR'],
                $row['NMOPERADOR'],
                $row['NRINSJURFORN'],
                $row['NMRAZSOCFORN'],
                $row['CDPRODUTO'],
                $row['NMPRODUTO'],
                $row['CDMARCA'],
                $row['NMMARCA'],
                $row['QTACOMPRAR']
            );
            array_push($rowsXlsx, $data);
        };

        return $rowsXlsx;
    }

    public function geraXlsxPedCol($rowsXlsx, $comprador, $fornecedor, $nameZipArchive)
    {

        $header = array(
            'REGIAO' => 'string',
            'NOME_REGIAO' => 'string',
            'VALIDADE_INI' => 'string',
            'VALIDADE_FIN' => 'string',
            'COMPRADOR' => 'string',
            'NOME_COMPRADOR' => 'string',
            'FORNECEDOR' => 'string',
            'NM_FORNECEDOR' => 'string',
            'PRODUTO' => 'string',
            'NOME_PR' => 'string',
            'MARCA' => 'string',
            'NOME_MARCA' => 'string',
            'QTDE' => '#,##0',
            'VRUNI' => 'string',
            'OBS_FORNECECOR' => 'string'
        );

        $writer = new \XLSXWriter();
        $styles = array('font-style' => 'bold');
        $nameArchive = date("Y-m-d_h-i-s") . '_' .  $comprador . '_' . $fornecedor;
        $writer->writeSheetHeader('Sheet1', $header, $styles);
        foreach ($rowsXlsx as $row) {
            $writer->writeSheetRow('Sheet1', $row);
        }
        $writer->writeToFile('/tmp/' . $nameArchive . '.xlsx');
        $nameReturn = $this->gerarZip($nameArchive, $nameZipArchive);

        return $nameReturn;
    }

    public function gerarZip($nameArchive, $nameZipArchive)
    {
        $zip = new ZipArchive();
        $filename = $nameZipArchive;
        $pathArchive = '/tmp/';

        if ($zip->open($pathArchive . $filename, ZipArchive::CREATE) === TRUE) {

            if ($handle = opendir($pathArchive)) {
                $zip->addFile($pathArchive . $nameArchive . '.xlsx', $nameArchive . '.xlsx');
                closedir($handle);
            }
            $zip->close();
            unlink($pathArchive . $nameArchive . '.xlsx');
        }
        $dataArchive = array(
            "NOMEARQUIVOZIP"   => $filename,
            "CAMINHOARQUIVOZIP"   => $pathArchive,
        );
        return $dataArchive;
    }

    public function downloadZipPedCol(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $file = $row["CAMINHOARQUIVOZIP"];
        $name = $row["NOMEARQUIVOZIP"];
        $nameArray = explode(".", $name);
        header('Accept-Ranges: none');
        header("Content-Type: application/zip");
        header("Content-Disposition: attachment; filename=" . $name);
        readfile($file . $name);
        unlink($row['CAMINHOARQUIVOZIP'] . $row['NOMEARQUIVOZIP']);
        exit;
    }

    public function setMnemonico($msg, $data)
    {
        $dtgeracao = date("d/m/Y");
        if (strpos($msg, 'CNPJ_CPF') !== false) {
            $msg = str_replace('CNPJ_CPF', substr($data["CDFORNECED"], 1), $msg);
        }
        if (strpos($msg, 'NM_FANTFORN') !== false) {
            $msg = str_replace('NM_FANTFORN', $data["NMFANTFORN"], $msg);
        }
        if (strpos($msg, 'NM_OPERADOR') !== false) {
            $msg = str_replace('NM_OPERADOR', $data["NMOPERADOR"], $msg);
        }
        if (strpos($msg, 'DT_GERACAO') !== false) {
            $msg = str_replace('DT_GERACAO', $dtgeracao, $msg);
        }
        if (strpos($msg, 'DT_VALIDADE') !== false) {
            $msg = str_replace('DT_VALIDADE', $data["VALIDADE_INI"] . " - " . $data["VALIDADE_FIN"], $msg);
        }
        return $msg;
    }

    public function emailFornecedor($fornecedor)
    {
        try {
            $query = "SELECT DSEMAILCOT FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
            $params = [
                "P_CDFORNECED" => $fornecedor
            ];
            $result = $this->supUtilities->crud($query, $params);
            $result = $result && $result[0] && $result[0]["DSEMAILCOT"] ? $result[0]["DSEMAILCOT"] : "";
            return $result;
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }

    public function getInitialParams(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'CDFILIAL',
                    'operator' => '=',
                    "value" => $this->environment->getCdFilial()
                )
            );

            $dataSet = $this->crudService->getFilteredDataSet("data_pedido_coleta", $conditions)->getRows();

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
